@@ -3,7 +3,6 @@ use Moose::Role;
 use HTTP::Request;
 use AWS::S3::ResponseParser;
 use MooseX::Types::URI qw(Uri);
-use URI::Escape qw/ uri_escape /;
 
 has 's3' => (
     is       => 'ro',
@@ -60,19 +59,8 @@ has '_uri' => (
             . '/'
         );
 
-		# note we add some extra exceptions to uri_escape to prevent
-		# encoding of things like "/", ":", "="
-        if ( $m->has_attribute('key') ) {
-            my $escaped_path = uri_escape( $self->key,"^A-Za-z0-9\-\._~\/:=" );
-
-            # if we have a leading slash in the key name we need to add *another*
-            # slash in the call to ->path to ensure it is retained
-            if ( $escaped_path =~ m!^/! && $self->s3->honor_leading_slashes ) {
-                $uri->path( '/'.$escaped_path )
-            } else {
-                $uri->path( $escaped_path )
-            }
-        }
+        $uri->path( $self->key )
+          if $m->has_attribute('key');
 
         $uri->query_keywords( $self->_subresource )
           if $m->has_attribute('_subresource');
